@@ -1,12 +1,11 @@
 from re import search
-from tkinter import Listbox
 
-from discord import Embed, Message, Interaction
+from discord import Embed, Message
 from discord.ext import commands
 
 from pycordViews import EasyModifiedViews
 
-from .utils.utils_message import utils_get_message
+from .utils.utils_message import utils_get_message, utils_autorised_mentions
 from .._utils import NoneType
 
 __all__ = [
@@ -24,7 +23,17 @@ __all__ = [
 ]
 
 
-async def dshell_send_message(ctx: Message, message=None, delete=None, channel=None, embeds=None, view=None) -> int:
+async def dshell_send_message(ctx: Message,
+                              message=None,
+                              delete=None,
+                              channel=None,
+                              global_mentions: bool = None,
+                              everyone_mention: bool = True,
+                              roles_mentions: bool = True,
+                              users_mentions: bool = True,
+                              reply_mention: bool = False,
+                              embeds=None,
+                              view=None) -> int:
     """
     Sends a message on Discord
     """
@@ -33,6 +42,7 @@ async def dshell_send_message(ctx: Message, message=None, delete=None, channel=N
         raise Exception(f'Delete parameter must be a number (seconds) or None, not {type(delete)} !')
 
     channel_to_send = ctx.channel if channel is None else ctx.channel.guild.get_channel(channel)
+    allowed_mentions = utils_autorised_mentions(global_mentions, everyone_mention, roles_mentions, users_mentions, reply_mention)
 
     if channel_to_send is None:
         raise Exception(f'Channel {channel} not found!')
@@ -54,12 +64,22 @@ async def dshell_send_message(ctx: Message, message=None, delete=None, channel=N
     sended_message = await channel_to_send.send(message,
                                                 delete_after=delete,
                                                 embeds=embeds,
+                                                allowed_mentions=allowed_mentions,
                                                 view=view)
 
     return sended_message.id
 
 
-async def dshell_respond_message(ctx: Message, message=None, content: str = None, delete=None, mention: bool = None, embeds=None):
+async def dshell_respond_message(ctx: Message,
+                                 message=None,
+                                 content: str = None,
+                                 global_mentions: bool = None,
+                                 everyone_mention: bool = True,
+                                 roles_mentions: bool = True,
+                                 users_mentions: bool = True,
+                                 reply_mention: bool = False,
+                                 delete=None,
+                                 embeds=None):
     """
     Responds to a message on Discord
     """
@@ -67,7 +87,8 @@ async def dshell_respond_message(ctx: Message, message=None, content: str = None
         raise Exception(f'Delete parameter must be a number (seconds) or None, not {type(delete)} !')
 
     respond_message = ctx if message is None else utils_get_message(ctx, message)  # builds a reference to the message (even if it doesn't exist)
-    mention_author = mention if mention is not None else False
+    autorised_mentions = utils_autorised_mentions(global_mentions, everyone_mention, roles_mentions, users_mentions, reply_mention)
+    mention_author = True if reply_mention else False
 
     from .._DshellParser.ast_nodes import ListNode
 
@@ -83,6 +104,7 @@ async def dshell_respond_message(ctx: Message, message=None, content: str = None
     sended_message = await respond_message.reply(
                                      content=str(content),
                                      mention_author=mention_author,
+                                     allowed_mentions=autorised_mentions,
                                      delete_after=delete,
                                      embeds=embeds)
 
