@@ -6,14 +6,16 @@ __all__ = [
 ]
 
 
-async def dshell_get_pastbin(ctx: Message, code: str):
+async def dshell_get_pastbin(ctx: Message, code: str) -> "ListNode":
     """
     Get a pastbin from a code snippet.
     """
     if not isinstance(code, str):
         raise Exception(f'Code must be a string, not {type(code)} !')
 
-    content = ''  # Initialize content to an empty string
+    from .._DshellParser.ast_nodes import ListNode
+
+    content = ListNode([])  # Initialize content to an empty string
 
     with get(f"https://pastebin.com/raw/{code}", stream=True, timeout=10) as response:
 
@@ -21,10 +23,6 @@ async def dshell_get_pastbin(ctx: Message, code: str):
             raise Exception(f"Failed to retrieve pastbin with code {code} !")
 
         for line in response.iter_lines(decode_unicode=True, chunk_size=512):
-            len_content = len(content)
-            if len_content < 4000 and len_content + len(line) <= 4000:
-                content += line + '\n'
-            else:
-                break
+            content.add(line)
 
     return content
