@@ -1,10 +1,9 @@
 from asyncio import sleep
 from re import findall
 from typing import TypeVar, Union, Any, Optional, Callable
-from random import choice
-from string import ascii_letters, digits
 from copy import deepcopy
-from pycordViews import SelectMenu, EasyModifiedViews
+from pycordViews import EasyModifiedViews
+from pycordViews.views.errors import CustomIDNotFound
 
 from discord import AutoShardedBot, Embed, Colour, PermissionOverwrite, Permissions, Guild, Member, Role, Message, Interaction, ButtonStyle
 from discord.ui import Button
@@ -63,10 +62,18 @@ class DshellInterpreteur:
             '__guild__': message.channel.guild.id,
             '__guild_name__': message.channel.guild.name,
             '__guild_id__': message.channel.guild.id,
+            '__guild_members__': ListNode([member.id for member in message.channel.guild.members]),
             '__guild_member_count__': message.channel.guild.member_count,
             '__guild_icon__': message.channel.guild.icon.url if message.channel.guild.icon else None,
             '__guild_owner_id__': message.channel.guild.owner_id,
             '__guild_description__': message.channel.guild.description,
+            '__guild_roles__': ListNode([role.id for role in message.channel.guild.roles]),
+            '__guild_roles_count__': len(message.channel.guild.roles),
+            '__guild_emojis__': ListNode([emoji.id for emoji in message.channel.guild.emojis]),
+            '__guild_emojis_count__': len(message.channel.guild.emojis),
+            '__guild_channels__': ListNode([channel.id for channel in message.channel.guild.channels]),
+            '__guild_channels_count__': len(message.channel.guild.channels),
+
         } if message is not None or not debug else {} # {} is used in debug mode, when ctx is None
         self.vars = vars if vars is not None else ''
         self.ctx: context = ctx
@@ -138,7 +145,14 @@ class DshellInterpreteur:
                     self.env[node.name.value] = build_permission(first_node.body, self)
 
                 elif isinstance(first_node, UiNode):
-                    self.env[node.name.value] = build_ui(first_node, self)
+                    # rebuild the UI if it already exists
+                    if node.name.value in self.env and isinstance(self.env[node.name.value], EasyModifiedViews):
+                        self.env[node.name.value] = rebuild_ui(first_node, self.env[node.name.value], self)
+                    else:
+                        self.env[node.name.value] = build_ui(first_node, self)
+
+                elif isinstance(first_node, LengthNode):
+                    self.env[node.name.value] = length(first_node)
 
                 else:
                     self.env[node.name.value] = eval_expression(node.body, self)
@@ -382,6 +396,19 @@ def regroupe_commandes(body: list[Token], interpreter: DshellInterpreteur, norma
 
     return list_tokens
 
+def length(variable : LengthNode) -> int:
+    """
+    Count characters or items in string/list
+    :param variable:
+    :return:
+    """
+    if not isinstance(variable.body, ListNode) and variable.body.type not in (DTT.STR, DTT.IDENT):
+        raise Exception(f'Length take string, ident or list, not {type(variable.body)} !')
+
+    if isinstance(variable.body, ListNode):
+        return len(variable.body)
+    else:
+        return len(variable.body.value)
 
 def build_embed(body: list[Token], fields: list[FieldEmbedNode], interpreter: DshellInterpreteur) -> Embed:
     """
@@ -424,6 +451,33 @@ def build_colour(color: Union[int, ListNode]) -> Union[Colour, int]:
     else:
         raise TypeError(f"Color must be an integer or a ListNode, not {type(color)} !")
 
+def build_ui_parameters(ui_node: UiNode, interpreter: DshellInterpreteur):
+    """
+    Builds the parameters for a UI component from the UiNode.
+    Can accept buttons and select menus.
+    :param ui_node:
+    :param interpreter:
+    :return:
+    """
+    for ident_component in range(len(ui_node.buttons)):
+        args_button: dict[str, list[Any]] = \
+        regroupe_commandes(ui_node.buttons[ident_component].body, interpreter, normalise=True)[0]
+        args_button.pop('--*', ())
+
+        code = args_button.pop('code', None)
+        style = args_button.pop('style', 'primary').lower()
+        custom_id = args_button.pop('custom_id', str(ident_component))
+
+        if not isinstance(custom_id, str):
+            raise TypeError(f"Button custom_id must be a string, not {type(custom_id)} !")
+
+        if style not in ButtonStyleValues:
+            raise ValueError(f"Button style must be one of {', '.join(ButtonStyleValues)}, not '{style}' !")
+
+        args_button['custom_id'] = custom_id
+        args_button['style'] = ButtonStyle[style]
+        args = args_button.pop('*', ())
+        yield args, args_button, code
 
 def build_ui(ui_node: UiNode, interpreter: DshellInterpreteur) -> EasyModifiedViews:
     """
@@ -435,26 +489,39 @@ def build_ui(ui_node: UiNode, interpreter: DshellInterpreteur) -> EasyModifiedVi
     """
     view = EasyModifiedViews()
 
-    for component in ui_node.buttons:
-        args_button: dict[str, list[Any]] = regroupe_commandes(component.body, interpreter, normalise=True)[0]
-        args_button.pop('--*', ())
-
-        code = args_button.pop('code', None)
-        style = args_button.pop('style', 'primary').lower()
-        if style not in ButtonStyleValues:
-            raise ValueError(f"Button style must be one of {', '.join(ButtonStyleValues)}, not '{style}' !")
-        args_button['style'] = ButtonStyle[style]
-        args = args_button.pop('*', ())
-        b = Button(*args, **args_button)
-
-        custom_id = ''.join(choice(ascii_letters + digits) for _ in range(20))
-        b.custom_id = custom_id
+    for args, args_button, code in build_ui_parameters(ui_node, interpreter):
+        b = Button(**args_button)
 
         view.add_items(b)
-        view.set_callable(custom_id, _callable=ui_button_callback, data={'code': code})
+        view.set_callable(b.custom_id, _callable=ui_button_callback, data={'code': code})
 
     return view
 
+def rebuild_ui(ui_node : UiNode, view: EasyModifiedViews, interpreter: DshellInterpreteur) -> EasyModifiedViews:
+    """
+    Rebuilds a UI component from an existing EasyModifiedViews.
+    :param view:
+    :param interpreter:
+    :return:
+    """
+    for args, args_button, code in build_ui_parameters(ui_node, interpreter):
+        try:
+            ui = view.get_ui(args_button['custom_id'])
+        except CustomIDNotFound:
+            raise ValueError(f"Button with custom_id '{args_button['custom_id']}' not found in the view !")
+
+        ui.label = args_button.get('label', ui.label)
+        ui.style = args_button.get('style', ui.style)
+        ui.emoji = args_button.get('emoji', ui.emoji)
+        ui.disabled = args_button.get('disabled', ui.disabled)
+        ui.url = args_button.get('url', ui.url)
+        ui.row = args_button.get('row', ui.row)
+        new_code = code if code is not None else view.get_callable_data(args_button['custom_id'])['code']
+        view.set_callable(args_button['custom_id'], _callable=ui_button_callback, data={'code': args_button.get('code', code)})
+
+    return view
+
+
 async def ui_button_callback(button: Button, interaction: Interaction, data: dict[str, Any]):
     """
     Callback for UI buttons.
@@ -464,7 +531,6 @@ async def ui_button_callback(button: Button, interaction: Interaction, data: dic
     :param data:
     :return:
     """
-    print(data)
     code = data.pop('code', None)
     if code is not None:
         local_env = {
